/******************************************************************************
Copyright (C) 2013 Linagora

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version, provided you comply with the Additional Terms applicable for OBM
connector by Linagora pursuant to Section 7 of the GNU Affero General Public
License, subsections (b), (c), and (e), pursuant to which you must notably (i)
retain the “Message sent thanks to OBM, Free Communication by Linagora”
signature notice appended to any and all outbound messages (notably e-mail and
meeting requests), (ii) retain all hypertext links between OBM and obm.org, as
well as between Linagora and linagora.com, and (iii) refrain from infringing
Linagora intellectual property rights over its trademarks and commercial
brands. Other Additional Terms apply, see <http://www.linagora.com/licenses/>
for more details.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License and
its applicable Additional Terms for OBM along with this program. If not, see
<http://www.gnu.org/licenses/> for the GNU Affero General   Public License
version 3 and <http://www.linagora.com/licenses/> for the Additional Terms
applicable to OBM connectors.
******************************************************************************/

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
Components.utils.import("resource://calendar/modules/calUtils.jsm");
Components.utils.import("resource://obm-extension/modules/UtilsOBM.jsm");
Components.utils.import("resource://obm-extension/modules/booleanChangesList.jsm");
Components.utils.import("resource://obm-extension/modules/subscriptionWidget.jsm");

const HTML_NS = "http://www.w3.org/1999/xhtml";
const MIN_RESULTS_PER_REQUEST = 20;

var UO_logger = Components.classes["@obm.org/calendar/logger;1"].createInstance(Components.interfaces.obmILogger);
var UO_writer = Components.classes["@obm.org/calendar/writer;1?type=unixfile"].getService(Components.interfaces.obmILoggerWriter);
UO_logger.setLevelWriter(UO_logger.LOAD_LEVEL, UO_writer);
UO_logger.setHeader("paginatedSubscription.js: ");

var userChanges = new BooleanChangesList();

var calMgr = cal.getCalendarManager();
var subscribedCalendars = calMgr.getCalendars({});

var obmCalMgr = UtilsOBM.getCalendarManager();

var checkAndGetResultsPerRequest = function() {
  let resultPref = UtilsOBM.getPrefSafe('extensions.obm.pagination.calendarList.resultsPerRequest', MIN_RESULTS_PER_REQUEST);
  if (resultPref >= MIN_RESULTS_PER_REQUEST) {
    return resultPref;
  }
  UtilsOBM.setPrefSafe('extensions.obm.pagination.calendarList.resultsPerRequest', MIN_RESULTS_PER_REQUEST);
  UO_logger.debug("extensions.obm.pagination.calendarList.resultsPerRequest is too low. Force setting it to the minimum of " + MIN_RESULTS_PER_REQUEST + ".");
  return MIN_RESULTS_PER_REQUEST;
};

var limitByConfig = checkAndGetResultsPerRequest();

var gotMutationObserver = false;
try {
  if ( MutationObserver ) {
    gotMutationObserver = true;
  }
} catch(e) {}

function fetchItems(method, limit, offset, pattern, callback) {
  var calSync = Components.classes["@obm.org/obm-sync-rest/calendarsync;1"].createInstance(Components.interfaces.obmIRCalendarSync);
  var opIdx = 0;
  var operations = [];

  operations[0] = ["Calendar", function(arg, limit, offset, pattern){calSync[method](arg, limit, offset, pattern)}];

  var listener = {
    onStart: function() {},
    onNewOperation: function(aOpName) {},
    onOperationDone: function(aSuccess, aResultData) {
      if (callback && aResultData) {
        callback(null, aResultData);
        callback = null;
      }
    },
    onStop: function(aRes, aMessage) {},
    onProgress: function(aIdx, aTotal, aType) {},
    onError: function(aMessage) {
      dump("EXCEPTION: " + aMessage + "\n");
      if ( callback ) {
        callback(aMessage);
        callback = null;
      }
    },
    nextOperation: function() {
      if (opIdx >= operations.length) {
        return;
      }

      try {
        operations[opIdx++][1](this, limit, offset, pattern);
      } catch(e) {
        listener.onError(e);
      }
    }
  };
  listener.nextOperation();
}

function buildUserTreeItems(calendars){
  var treeItems = [];

  for (var i = 0; i < calendars.length; i++) {
    var child = calendars.queryElementAt(i, Components.interfaces.obmIRCalendarInfo);
    var changes = userChanges.getChanges();
    var subs = changes[child.uid] ?
                changes[child.uid].value : (obmCalMgr.getObmCalendar(child.uid) != null);

    var treeItem = document.createElement("treeitem");
    var treerow = treeItem.appendChild(document.createElement("treerow"));
    treerow.setAttribute("id", "_" + child.uid);
    treerow.appendChild(document.createElement("treecell")).setAttribute("properties", (!child.write ? "readOnly":"readWrite"));
    treerow.appendChild(document.createElement("treecell")).setAttribute("label", child.lastname + " " + child.firstname);
    treerow.appendChild(document.createElement("treecell")).setAttribute("label", child.mail);
    var cb = treerow.appendChild(document.createElement("treecell"));
    cb.setAttribute("value", subs);
    treeItems.push(treeItem);
  }
  return treeItems;
}

function buildResourceTreeItems(calendars) {
  var subs = false;
  var treeItems = [];
  for (var i = 0; i < calendars.length; i++) {
    var child = calendars.queryElementAt(i, Components.interfaces.obmIRResourceInfo);

    // The server might send resources with read=false and write=false because the SQL query is not properly written
    // to exclude these. Until this is fixed server-side, we must filter the resources here.
    if (!child.read) {
      continue;
    }

    var changes = userChanges.getChanges();
    var subscribed = changes[child.mail] ?
                      changes[child.mail].value :
                      _resourceIsSubscribed(child.mail) ? "true" : "false";

    var treeItem = document.createElement("treeitem");
    var treerow = treeItem.appendChild(document.createElement("treerow"));
    treerow.setAttribute("id", "_" + child.id);
    treerow.setAttribute("properties", child.mail);
    treerow.appendChild(document.createElement("treecell")).setAttribute("properties", (!child.write ? "readOnly":"readWrite"));
    treerow.appendChild(document.createElement("treecell")).setAttribute("label", child.name);
    treerow.appendChild(document.createElement("treecell")).setAttribute("label", child.description);
    var cb = treerow.appendChild(document.createElement("treecell"));
    cb.setAttribute("value", subscribed);
    treeItems.push(treeItem);
  }
  return treeItems;
};

function _resourceIsSubscribed(aResInfoMail) {
  for each(var calendar in subscribedCalendars) {
    if (calendar.type == "ics" && calMgr.getCalendarPref_(calendar, "X-OBM-RESOURCE-EMAIL") == aResInfoMail) {
      return calendar;
    }
  }
  return null;
}

function removeCalendars (calendars) {
  function removeCalendar(calendar) {
    calMgr.unregisterCalendar(calendar);
    calMgr.deleteCalendar(calendar);
  };
  
  for each ( var calendar in subscribedCalendars ) {
    if ( calendar.type == 'obm' ) {
      var uri = UtilsOBM.getCalendarOBMName(calendar);
      if ( uri.split(";").pop() in calendars ) {
        removeCalendar(calendar);
      }
    } else if ( calendar.type == 'ics' ) {
      var email = calMgr.getCalendarPref_(calendar, "X-OBM-RESOURCE-EMAIL");
      if ( email && (email in calendars) ) {
        removeCalendar(calendar);
      }
    }
  }
};


function addCalendars(calendars) {
  
  function addCalendar(checkedCalendarInfos) {
    let calendarName = checkedCalendarInfos.label;
    let email = checkedCalendarInfos.email;
    let color = UtilsOBM.getRandomColor();
    let readOnly = checkedCalendarInfos.readOnly;
    let calendar = obmCalMgr.getObmCalendar(checkedCalendarInfos.id);
    if (calendar) {
      // update infos
      obmCalMgr.updateObmCalendar(calendar, checkedCalendarInfos.id, calendarName, email, readOnly);
    } else {
      obmCalMgr.subscribeObmCalendar(checkedCalendarInfos.id, calendarName, email, readOnly);
    }
  };
  
  function addResourceCalendar(calendarInfos) {
    let name = "[" + UtilsOBM.getLocalizedMessage("obm.resource.label") + "] " + calendarInfos.label;
    let email = calendarInfos.email;
    let color = UtilsOBM.getRandomColor();
    let readOnly = calendarInfos.readOnly == "readOnly";
    var uri = UtilsOBM.createResourceUri(email);
	var newCalendar = calMgr.createCalendar("ics", uri);
    calMgr.registerCalendar(newCalendar);
	newCalendar.name = name;
	newCalendar.setProperty("X-OBM-DEFAULT-NAME", name);
	newCalendar.setProperty("X-OBM-RESOURCE-EMAIL", email);
	newCalendar.setProperty("X-OBM-EMAIL", email);
	newCalendar.setProperty("color", color.toString());
  };
  
  calendars.forEach(function(calendarInfos) {
    if ( calendarInfos.isResource ) {
      addResourceCalendar(calendarInfos) ;
    }else {
      addCalendar(calendarInfos);
    }
  });
};

function saveSubscriptionSettings() {
  var changes = userChanges.getChanges();
  var added = [], removed = {};
  for (var name in changes) {
    if ( changes[name].value === false ) {
      removed[name] = true;
    } else {
      changes[name].metadata.id = name;
      added.push(changes[name].metadata);
    }
  }
  if ( Object.keys(removed).length ) {
      removeCalendars(removed);
  }
  if ( added.length ) {
    addCalendars(added);
  }
};

function initUsersSubscription() {
  var dom = document.getElementById('usertab');
  var userWidget = new SubscriptionWidget(dom, buildUserTreeItems, limitByConfig, fetchItems, 'getSubscribleCalendars');
  userWidget.setTreeObserver(gotMutationObserver ? MutationObserver : null, function(mutations) {
    mutations.forEach(function(mutation) {
      var _calendarId = mutation.target.parentNode.id;
      var md = {};
      if ( !_calendarId ) {
        return ;
      }
      var calendarId = _calendarId.substr(1);
      var emailCell = mutation.target.previousSibling;
      md.email = emailCell.getAttribute('label');
      var labelCell = emailCell.previousSibling;
      md.label = labelCell.getAttribute('label');
      var roCell = labelCell.previousSibling;
      md.readOnly = roCell.getAttribute('properties') === 'readOnly' ? true : false;
      var checked = mutation.target.getAttribute('value') === 'true' ? true : false;
      userChanges.setChange(calendarId, checked, md);
    });
  });
  userWidget.search();
};

function initResourcesSubscription() {
  var dom = document.getElementById('resourcetabpanel');
  var resourcesWidget = new SubscriptionWidget(dom, buildResourceTreeItems, limitByConfig, fetchItems, 'getSubscribleResources');

  resourcesWidget.setTreeObserver(gotMutationObserver ? MutationObserver : null, function(mutations) {
    mutations.forEach(function(mutation) {
      var _calendarId = mutation.target.parentNode.id;
      if ( !_calendarId ) {
        return ;
      }
      var calendarId = _calendarId.substr(1);
      var md = {
        isResource: true,
        email: mutation.target.parentNode.getAttribute('properties')
      };
      var descCell = mutation.target.previousSibling;
      md.description = descCell.getAttribute('label');
      var labelCell = descCell.previousSibling;
      md.label = labelCell.getAttribute('label');
      var roCell = labelCell.previousSibling;
      md.readOnly = roCell.getAttribute('properties') === 'readOnly' ? true : false;
      var checked = mutation.target.getAttribute('value') === 'true' ? true : false;
      userChanges.setChange(md.email, checked, md);
    });
  });
  resourcesWidget.search();
};

function bindTabActivation() {
  var resourceTab = document.getElementById('resourceintabs');
  var handler = function() {
    resourceTab.removeEventListener('command',handler,false);
    initResourcesSubscription();
  };
  resourceTab.addEventListener('command',handler, false);
};

function onLoad() {
  bindTabActivation();
  initUsersSubscription();
}

window.addEventListener("load", onLoad, false);
