/******************************************************************************
Copyright (C) 2011-2012 Linagora

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version, provided you comply with the Additional Terms applicable for OBM
connector by Linagora pursuant to Section 7 of the GNU Affero General Public
License, subsections (b), (c), and (e), pursuant to which you must notably (i)
retain the “Message sent thanks to OBM, Free Communication by Linagora”
signature notice appended to any and all outbound messages (notably e-mail and
meeting requests), (ii) retain all hypertext links between OBM and obm.org, as
well as between Linagora and linagora.com, and (iii) refrain from infringing
Linagora intellectual property rights over its trademarks and commercial
brands. Other Additional Terms apply, see <http://www.linagora.com/licenses/>
for more details.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License and
its applicable Additional Terms for OBM along with this program. If not, see
<http://www.gnu.org/licenses/> for the GNU Affero General   Public License
version 3 and <http://www.linagora.com/licenses/> for the Additional Terms
applicable to OBM connectors.
******************************************************************************/


Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

function calOBMCalendar() {
    try {
        Components.utils.import("resource://obm-extension/modules/UtilsOBM.jsm");
        Components.utils.import("resource://obm-extension/modules/EmailNotificationDialog.jsm");
        Components.utils.import("resource://obm-extension/modules/EventComparator.jsm");
        Components.utils.import("resource://calendar/modules/calUtils.jsm");
    } catch (e) {
        Components.utils.reportError(e);
    }

	this.wrappedJSObject = this;
	// a reference to the underlying native calendar storage
	this.mLocalCalendar = null;
	// a reference to our calOBMObserver, to know if it has been added or not
	// into the mLocalCalendar object
	this.mObserver = null;
	// These ICS properties are really transient - we need them to know where an
	// update was from, but they should not be serialized
	this.eventContextProps = [
		"X-OBM-ORIGIN",
		"X-OBM-DOMAIN",
		"X-OBM-DOMAIN-UUID",
		"METHOD",
			];
	try {
		//Init logger
		this.logger = Cc["@obm.org/calendar/logger;1"].createInstance(Ci.obmILogger);
		var writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Ci.obmILoggerWriter);
		this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
		this.logger.setHeader("calOBMCalendar: ");
		this.logger.debug("new calOBMCalendar()", false, false);

		var scriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].createInstance(Ci.mozIJSSubScriptLoader);

		if (typeof(Calendar_Event_Synchronizer) == "undefined") {
			scriptLoader.loadSubScript("chrome://obm-extension/content/synchronizer/Calendar_Event_Synchronizer.js");
		}
		this.mEventSynchronizer = new Calendar_Event_Synchronizer();

		if (typeof(Calendar_Task_Synchronizer) == "undefined") {
			scriptLoader.loadSubScript("chrome://obm-extension/content/synchronizer/Calendar_Task_Synchronizer.js");
		}
		this.mTaskSynchronizer = new Calendar_Task_Synchronizer();

	} catch(e) {
        Components.utils.reportError(e);
	}

    this._ensureCalendarACLEntry();
}

calOBMCalendar.prototype = {
	//
	// nsISupports interface
	//
    QueryInterface: XPCOMUtils.generateQI([
       Ci.calICalendar,
       Ci.calICalendarACLManager,
       Ci.calICalendarProvider,
       Ci.nsIRunnable,
       Ci.calISchedulingSupport
    ]),

    classDescription: "Calendar OBM Provider",
    classID: Components.ID("{86cb9e4f-44c6-4734-9aed-a22795e92b1e}"),
    contractID: "@mozilla.org/calendar/calendar;1?type=obm",

	//
	// nsIRunnable
	//
	run: function() {
		this.refresh();
	},

	//
	// calICalendar interface
	//
	mID: null,
	get id() {
		return this.mID;
	},
	set id(id) {
		if (this.mID) {
			throw Cr.NS_ERROR_ALREADY_INITIALIZED;
		}
		this.mLocalCalendar.id = id;
		dump("this.mLocalCalendar.id:" + this.mLocalCalendar.id + "\n");
		return (this.mID = id);
	},

	// attribute AUTF8String name;
	get name() {
		var name = cal.getCalendarManager().getCalendarPref_(this, "NAME");
		if (!this.mLocalCalendar.name) {
			//mLocalCalendar.name is needed for light 1.0
			this.mLocalCalendar.name = name;
		}
		return name;
	},
	set name(name) {
		cal.getCalendarManager().setCalendarPref_(this, "NAME", name);
		return (this.mLocalCalendar.name = name);
	},

	// readonly attribute AUTF8String type;
	get type() {
		return "obm";
	},

	get superCalendar() {
		return (this.mSuperCalendar ? this.mSuperCalendar.superCalendar: this);
	},
	set superCalendar(val) {
		return (this.mSuperCalendar = val);
	},

	// attribute nsIURI uri;
	mUri: null,
	get uri() {
	    return this.mURI;
	},
	set uri(aURI) {
		try {
			if (!this.mURI) {
				dump("set uri" + aURI.spec + "\n");
				this.mURI = aURI;
				this.mOBMName = this.mURI.spec.split(";").pop();
				this.initLocalCalendar(aURI);
			}
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// attribute boolean readOnly;
	mReadOnly: null,
	get readOnly() {
        if (this.synchronizing) {
            return false;
        } else {
            return UtilsOBM.getPrefSafe("extensions.obm.changes.calICalendar." + this.mOBMName + ".readOnly", false);
        }
	},
	set readOnly(bool) {
		// nothing, user must not be able to change it
	},

    get aclManager() {
        return this.QueryInterface(Components.interfaces.calICalendarACLManager);
    },

    mACLEntry: null,
    _ensureCalendarACLEntry: function initCalendarACLEntry() {
        if (this.mACLEntry) {
            return;
        }
        var calendar = this;
        this.mACLEntry = {
            aclManager: this,
            hasAccessControl: true,
            get userIsOwner() {
                var login = {};
                UtilsOBM.getOBMCredentials(login, {});
                if (login.value) {
                    return (login.value == calendar.getProperty("X-OBM-UID"));
                } else {
                    // No login? We can't say for sure, lets assume yes.
                    return true;
                }
            },
            get userCanAddItems() {
                return calendar.getProperty("X-OBM-WRITABLE") !== false;
            },
            get userCanDeleteItems() {
                return calendar.getProperty("X-OBM-WRITABLE") !== false;
            },

            _getIdentities: function _getIdentities(aCount) {
                let returnIdentities = [];
                let identity = calendar.getProperty("imip.identity");
                if (identity) {
                    returnIdentities.push(identity);
                }

                aCount.value = returnIdentities.length;
                return returnIdentities;
            },

            getUserAddresses: function getUserAddresses(aCount) {
                let identities = this._getIdentities(aCount);
                let addresses = [ id.email for each (id in identities) ];
                return addresses;
            },

            getUserIdentities: function getUserIdentities(aCount) {
                return this._getIdentities(aCount);
            },

            getOwnerIdentities: function getOwnerIdentities(aCount) {
                return this._getIdentities(aCount);
            },

            // No need to refresh, this is done on sync
            refresh: function refresh() {}
        };
    },
    getCalendarEntry: function getCalendarEntry(aCalendar, aListener) {
        cal.ASSERT(aCalendar.type == "obm", "Attempt to retrieve ACL entry on wrong calendar " + aCalendar.type);
        if (aCalendar.id != this.id) {
            // Each OBM calendar has its own acl manager, just ask the right one
            return aCalendar.aclManager.getCalendarEntry(aCalendar, aListener);
        }

        this._ensureCalendarACLEntry();
        aListener.onOperationComplete(aCalendar, Components.results.NS_OK,
                                      Components.interfaces.calIOperationListener.GET,
                                      aCalendar.id,
                                      this.mACLEntry);
    },

    getItemEntry: function getItemEntry(aItem) {
        cal.ASSERT(aItem.calendar.type == "obm", "Attempt to retrieve ACL entry on wrong calendar " + aItem.calendar.type);
        if (aItem.calendar.id != this.id) {
            // Each OBM calendar has its own acl manager, just ask the right one
            return aItem.calendar.aclManager.getItemEntry(aItem);
        }

        this._ensureCalendarACLEntry();

        let aclEntry = {
            calendarEntry: this.mACLEntry,
            userCanViewDateAndTime: true,
            get userCanViewAll() {
                // OBM uses the privacy field to denote these events
                return (aItem.privacy != "PRIVATE");
            },

            // OBM doesn't provide specific rights for modifications per item.
            // Until then, just check if the user can delete the items (write support)
            get userCanModify() {
                return this.calendarEntry.userCanDeleteItems;
            },
            get userCanRespond() {
                return this.calendarEntry.userCanDeleteItems;
            }
        };

        return aclEntry;
    },

	// readonly attribute boolean canRefresh;
	get canRefresh() {
		return false;
	},

	// nsIVariant getProperty(in AUTF8String aName);
	getProperty: function(aName) {
        switch (aName) {
            case "imip.identity.disabled":
                return true;
            case "organizerId":
                return "mailto:" + this.getProperty("X-OBM-EMAIL");
            case "capabilities.privacy.values":
                if (UtilsOBM.capabilitySupported("CONFIDENTIAL_EVENTS")) {
                    return ["PUBLIC", "PRIVATE", "CONFIDENTIAL"];
                } else {
                    return ["PUBLIC", "PRIVATE"];
                }
            case "itip.copyProperties":
                return this.eventContextProps;
        }
        return this.mLocalCalendar.getProperty(aName);
	},
	// void setProperty(in AUTF8String aName, in nsIVariant aValue);
	setProperty: function(aName, aValue) {
		this.mLocalCalendar.setProperty(aName, aValue);
	},
	// void deleteProperty(in AUTF8String aName);
	deleteProperty: function(aName) {
		cal.getCalendarManager().deleteCalendarPref_(this, aName);
	},
	// void addObserver( in calIObserver observer )
	addObserver: function(aObserver) {
		try {
			if (!this.mObserver) { // to be sure to have our observer. Not sure it is necessary
				this.mObserver = new calOBMObserver(this, this.mOBMName, this.mEventSynchronizer, this.mTaskSynchronizer);
				this.mLocalCalendar.addObserver(this.mObserver);
			}
			// and add lightning observer
			this.mLocalCalendar.addObserver(aObserver);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void removeObserver( in calIObserver observer );
	removeObserver: function(aObserver) {
		try {
			this.mLocalCalendar.removeObserver(aObserver);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void addItem( in calIItemBase aItem, in calIOperationListener aListener );
	addItem: function(aItem, aListener) {
		this.logger.debug("addItem", false, false);
		try {
			if (this.readOnly) {
				throw Ci.calIErrors.CAL_IS_READONLY;
			}

			var addedThroughICS = false;
			var self = this;
			var checkDomainUuid = false, sameDomainUuid = false;
			var flagEventAsInternal = function(event) {
				event.setProperty('X-OBMEXT-PARTICIPATION-CHANGED', 'YES');
				event.setProperty('X-OBMEXT-IS-INTERNAL', 'YES');
				self.logger.debug(" set properties X-OBMEXT-IS-INTERNAL & X-OBMEXT-PARTICIPATION-CHANGED", false, false);
			};
			// check if the event is added through the email blue bar
			if (aItem instanceof Ci.calIEvent
				&& ( aItem.hasProperty("X-MOZ-RECEIVED-SEQUENCE") || aItem.hasProperty("X-MOZ-RECEIVED-DTSTAMP") ) ) {
				addedThroughICS = true;
				// if the ics received got an obm server uuid and we got it too, use it so
				// we'll know without any doubt if the event is internal
				if ( UtilsOBM.domainUuid.get() && aItem.hasProperty("X-OBM-DOMAIN-UUID") ) {
					checkDomainUuid = true;
					if ( UtilsOBM.domainUuid.get() == aItem.getProperty("X-OBM-DOMAIN-UUID") ) {
						this.logger.debug("addItem: domain uuid and ics domain uuid are the same", false, false);
						sameDomainUuid = true;
					}
				}
			}

			if ( ! aItem.isMutable ) {
				aItem = aItem.clone();
			}

			// the event has been added using Lightning GUI :
			// we set the X-OBMEXT-IS-INTERNAL flag
			if (cal.isEvent(aItem) && !addedThroughICS ) {
				aItem.setProperty('X-OBMEXT-IS-INTERNAL', 'YES');
			}

			if (cal.isEvent(aItem) 	
				&& (checkDomainUuid ||aItem.hasProperty('X-OBM-DOMAIN')) ) {
				if ( checkDomainUuid ) {
					if ( sameDomainUuid ) {
						flagEventAsInternal(aItem);
					}
				} else {
					// let's check if this is an internal OBM event
					// if this is it, this is probably an event added when
					// the user "accept" or "decline" a participation from
					// the mail sent for the participation
					// note: we cannot do this directly into the observer, into onAddedItem
					// because onAddedItem is called when adoptItem is called
					// by our synchronizer during the import of new events
					var obmDomain =  aItem.getProperty('X-OBM-DOMAIN');
					let authDomain = UtilsOBM.getPrefSafe("extensions.obm.server.domain", "");
					if (authDomain != '' && authDomain == obmDomain) {
						// this is an internal event, already created into our server
						// we should then notify as modified, not as added.
						// we mark our event. the property will be removed later
						flagEventAsInternal(aItem);
					}
				}
			}

            // If the item has no organizer and is internal, we need to add one
            let organizerId = this.getProperty("organizerId");
            if (aItem.getProperty("X-OBMEXT-IS-INTERNAL") == "YES" && !aItem.organizer && organizerId) {
                var organizer = cal.createAttendee();
                organizer.id = organizerId;
                organizer.isOrganizer = true;
                organizer.commonName = this.getProperty("organizerCN");
                aItem.organizer = organizer;
            }
			/*
			// try to add organizer to attendees of the event
			if ( aItem.organizer && aItem.organizer.id ) {
				var organizerFound = false;
				var atts = aItem.getAttendees({});
				for ( var i in atts ) {
					this.logger.debug("attendees...",false,false);
					this.logger.debug(atts[i].id,false,false);
					this.logger.debug(atts[i].commonName,false,false);
					if ( atts[i].id == aItem.organizer.id ) {
						organizerFound = true;
					}
				}
				// add organizer to the attendees
				if ( !organizerFound ) {
					aItem = aItem.clone();
					var att = aItem.organizer.clone();
					att.isOrganizer = true;
					aItem.addAttendee(att);
				}
				this.logger.debug("organizer...",false,false);
				this.logger.debug(aItem.organizer.id,false,false);
				this.logger.debug(aItem.organizer.commonName,false,false);
			}
			*/
			if (!aItem.isMutable) {
				aItem = aItem.clone();
			}
			// Ignore returned context properties, it's the 'stripping' part we
			// are after
			this._stripAndReturnContextProperties(aItem);
			aItem.makeImmutable();
			this.mLocalCalendar.addItem(aItem, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void adoptItem( in calIItemBase aItem, in calIOperationListener aListener );
	adoptItem: function(aItem, aListener) {
		try {
			this.logger.debug("adoptItem", false, false);
			if (this.readOnly) {
				throw Ci.calIErrors.CAL_IS_READONLY;
			}
			// Ignore returned context properties, it's the 'stripping' part we
			// are after
			this._stripAndReturnContextProperties(aItem);
			this.mLocalCalendar.adoptItem(aItem, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	_canModifyItem: function(aNewItem, aOldItem, callback) {
		if (!aOldItem) {
			// no old item found?  should be using addItem, then.
			// let calendar report the error
			return callback(false);
		}
		var listener = {
			onOperationComplete: function(aCalendar, aStatus, aOperationType, aId, aDetail) {
				if (!this.storedOldItem) {
					// no old item found in calendar ?  should be using addItem, then.
					// let calendar report the error
					return callback(false);
				}

				aOldItem = aOldItem.parentItem;

				if (aOldItem.generation != this.storedOldItem.generation) {
					return callback(false);
				}
				return callback(true);
			},
			onGetResult: function(aCalendar, aStatus, aItemType, aDetail, aCount, aItems) {
				if (aCount > 0) {
					this.storedOldItem = aItems[0];
				}
			}
		};
		listener.storedOldItem = null;
		this.mLocalCalendar.getItem(aOldItem.id, listener);
	},

    /**
     * Propagate the sequence number to the given master item and all its
     * exceptions. This function is used to make sure exceptions and master
     * item have the same sequence number, as there is only one sequence number
     * per item for OBM items.
     *
     * @param aItem         The item to apply to. This MUST be the master item.
     * @param aNewSequence  The new sequence number to set.
     */
    _propagateSequence: function(aItem, aNewSequence) {
        let self = this;
        UtilsOBM.retainStamps(aItem, function(aItem) {
            if (aNewSequence) {
                aItem.setProperty("SEQUENCE", aNewSequence);
            } else {
                aItem.deleteProperty("SEQUENCE");
            }
            var rec = aItem.recurrenceInfo;
            if (rec) {
                var exceptions = rec.getExceptionIds ({});
                if (exceptions.length > 0) {
                    for each (exid in exceptions) {
                        let ex = rec.getExceptionFor(exid);
                        if (aNewSequence) {
                            ex.setProperty("SEQUENCE", aNewSequence);
                        } else {
                            ex.deleteProperty("SEQUENCE");
                        }
                    }
                }
            }
        });
    },

	// void modifyItem( in calIItemBase aNewItem, in calIItemBase aOldItem,
	//                  in calIOperationListener aListener );
	modifyItem: function(aNewItem, aOldItem, aListener) {
		this.logger.debug("modifyItem (" + this.mOBMName + ")", false, false);
		this.logger.debug("Synchronizing ? " + this.synchronizing,false,false);
		try {
			if (this.readOnly) {
				//this.logger.debug("modifyItem: error, readonly", false, false);
				throw Ci.calIErrors.CAL_IS_READONLY;
			}
			var self = this;
			this._canModifyItem(aNewItem, aOldItem, function (canModify) {
				//self.logger.debug("callback of _canModifyItem", false, false);
				if (!canModify) {
					var stringBundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
						.getService(Components.interfaces.nsIStringBundleService)
						.createBundle("chrome://obm-extension/locale/errors_warnings.properties");
					var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
						.getService(Components.interfaces.nsIPromptService);
					promptService.alert(null,stringBundle.GetStringFromName("obm.modify.alert.title"),stringBundle.GetStringFromName("obm.modify.alert.content"));
				} else {
				    // test if this update come from an ICS mail
				    var fromIcs = aNewItem.hasProperty("METHOD");
					self.logger.debug("update from ICS ? "+fromIcs,false,false);
				    
					// if it's an ICS or an obm-sync we are in blind mode
				    // otherwise test if we're a recurrent evt
					if ( !self.synchronizing && !fromIcs && aNewItem.parentItem  == aNewItem && aNewItem.recurrenceInfo ) {
					  self.logger.debug("We're in a recurrent event",false,false);
					  var hasBeenReinitialized = UtilsOBM.localEventIsReinitialized(aOldItem, aNewItem);
					  self.logger.debug("test on major change (that requires a reinit of the recurrence)",false,false);
					  if ( hasBeenReinitialized ) {
						self.logger.debug("=> I should remove exceptions",false,false);
                       var exIds = aNewItem.recurrenceInfo.getExceptionIds({});
                       for each(exId in exIds ) {
                         self.logger.debug("Removing exception "+exId,false,false);
                         aNewItem.recurrenceInfo.removeExceptionFor(exId);
                       }
						var exDates = UtilsOBM.getExdates(aNewItem,true);
						for each ( var exDate in exDates ) {
						  self.logger.debug("restoring from exdate "+exDate,false,false);
						  aNewItem.recurrenceInfo.restoreOccurrenceAt(exDate);
						}
                     }

                     //participationStatus
                     var ownerName = cal.getCalendarManager().getCalendarPref_(aNewItem.calendar, "X-OBM-EMAIL"),
                         ownerId = "MAILTO:" + ownerName;

                     // Check if an exdate was added where we are an attendee.
                     // If so, change participation status instead
                     if (self.isInvitation(aOldItem)) {
                       UtilsOBM.calculateChanges(
                         UtilsOBM.getExdates(aOldItem, true),
                         UtilsOBM.getExdates(aNewItem, true),
                         function keyfunc(dt) dt.icalString,
                         function onadd(dt) {
                           // Added exdate, we need to remove paricipation status instead.
                           aNewItem.recurrenceInfo.restoreOccurrenceAt(dt);

                           var occ = aNewItem.recurrenceInfo.getOccurrenceFor(dt);
                           var att = occ.getAttendeeById(ownerId);
                           occ.removeAttendee(att);
                           att = att.clone();
                           att.participationStatus = "DECLINED";
                           occ.addAttendee(att);
                           aNewItem.recurrenceInfo.modifyException(occ,true);
                         }
                       );
                     }

                     // If the participation status for our user has changed,
                     // do so on all exceptions too.
                     var localUserOld = aOldItem.getAttendeeById(ownerId);
                     var localUserNew = aNewItem.getAttendeeById(ownerId);
                     if ( localUserOld && localUserNew ) {
                       if ( localUserOld.participationStatus != localUserNew.participationStatus ) {
                         self.logger.debug("Should change participationStatus on exceptions ",false,false);
                         var exIds = aNewItem.recurrenceInfo.getExceptionIds({});
                         for each(exId in exIds ) {
                           self.logger.debug("Changing exception "+exId,false,false);
                           var ex = aNewItem.recurrenceInfo.getExceptionFor(exId);
                           ex = ex.clone();
                           var att = ex.getAttendeeById(ownerId);
                           if ( !att ) {
                             continue;
                           }
                           var newAtt = att.clone();
                           ex.removeAttendee(att);
                           newAtt.participationStatus = localUserNew.participationStatus;
                           ex.addAttendee(newAtt);
                           aNewItem.recurrenceInfo.modifyException(ex,true);
                         }
                       }
                     }
                   }
                   
                   try {
						if (!aNewItem.isMutable) {
							self.logger.debug("aNewItem is not mutable, cloning it", false, false);
							aNewItem = aNewItem.clone();
						}
						self.logger.debug("Stripping properties", false, false);
						var contextProperties = self._stripAndReturnContextProperties(aNewItem);

						if ( self.synchronizing || fromIcs ) {
							// save the DtStamp... http://ci-obm.linagora.com/fisheye/browse/lightning/calendar-js/calItemBase.js?hb=true#to424
							aNewItem.setProperty("DTSTAMP", aNewItem.getProperty("LAST-MODIFIED"));
							aNewItem.setProperty("LAST-MODIFIED", aNewItem.getProperty("LAST-MODIFIED"));

							self.logger.debug("Setting event dtstamp to " + aNewItem.getProperty("LAST-MODIFIED"),false,false);
						} else {
							self.logger.debug("Update is not from obm-sync and not from ICS => don't set event DTSTAMP",false,false);
						}
						self.logger.debug("Modifying aNewItem", false, false);
                        var opListener = {
                            onGetResult: function() {},
                            onOperationComplete: function(calendar, status, opType, id, detail) {
                                if (Components.isSuccessCode(status)) {
                                    self.logger.debug("Update successful, registering modifications", false, false);
                                    self._registerEventModificationAndPromptForNotification(aNewItem,
                                      aOldItem, contextProperties);
                                } else {
                                    self.logger.debug("modifyItem error: " + status+", details: "+detail,true,true);
                                }
                                
                                if ( aListener && ( "onOperationComplete" in aListener ) ) {
                                    self.logger.debug("calling aListener.onOperationComplete",false,false);
                                    aListener.onOperationComplete.apply(aListener, arguments);
                                }
                            }
                        };

                        // So we can better control what is being changed, move
                        // to the parent item now.
                        let parentItem = aNewItem.parentItem.clone();
                        if (aNewItem.parentItem != aNewItem) {
                            parentItem.recurrenceInfo.modifyException(aNewItem, true);
                        }

                        // Before we send the update, make sure the sequence
                        // number is advanced on all instances, using the
                        // sequence of the original item passed. This can be
                        // the master item or an occurrence
                        self._propagateSequence(parentItem, aNewItem.getProperty("SEQUENCE"));


                        // If this is an instance, we've updated it in
                        // _propagateSequence without updating the reference we
                        // have for aNewItem.
                        if (aNewItem.recurrenceId) {
                            aNewItem = parentItem.recurrenceInfo.getExceptionFor(aNewItem.recurrenceId);
                        }

                        // Now send the modification
                        let updateResult = self.mLocalCalendar.modifyItem(parentItem, aOldItem.parentItem, opListener);
// 						if (updateResult == null) {
// 							self.logger.debug("Update successful, registering modifications", false, false);
// 							self._registerEventModificationAndPromptForNotification(aNewItem,
// 									aOldItem, contextProperties);
// 						}
						//self.logger.debug("modifyItem on mLocalCalendar ok ", false, false);
					} catch(e) {
						//self.logger.debug("modifyItem on mLocalCalendar: error ", false, false);
						self.logger.error(e, true, false);
					}
				}
			});
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	_registerEventModificationAndPromptForNotification: function cOC_registerEventModificationAndPromptForNotification(aNewItem, aOldItem, contextProperties) {
		var updateContext = new UpdateContext(contextProperties);
		if (updateContext.type == UpdateContext.OBM_SYNC) {
			this.logger.debug("The change comes from obm-sync, don't do anything", false, false);
			return;
		}
		this.logger.debug("_registerEventModificationAndPromptForNotification(" + aNewItem.title + " <- " + aOldItem.title + ")", false, false);
		if (aNewItem instanceof Ci.calIEvent) {
			this.logger.debug("This item is an event", false, false);
			var eventChanges = new EventChanges(aNewItem, aOldItem, updateContext, this.logger);
			this.logger.debug("Computing event changes between new event " + aNewItem + " and old event " + aOldItem, false, false);
			var changeType = eventChanges.compute();
			this.logger.debug("Found the following changes between events: " + changeType.types.join(", "), false, false);
			this.logger.debug("Prompting for notification", false, false);
			this._promptForNotification(aNewItem, aOldItem, changeType);
			this.logger.debug("Registering event modification", false, false);
			this._registerEventModification(aNewItem, changeType);
		} else if (aNewItem instanceof Ci.calITodo) {
			this.logger.debug("_registerEventModificationAndPromptForNotification: " +
					"this item is a 'todo'", false, false);
			this.mTaskSynchronizer.notifyItemModified(this, aNewItem);
		}
	},

	_promptForNotification: function cOC_promptForNotification(aNewItem, aOldItem, changeType) {
		if (changeType.isType(EventChangeType.USER_EVENT_MAJOR_CHANGES) ||
				changeType.isType(EventChangeType.USER_EVENT_MINOR_CHANGES_WITH_UPDATED_ATTENDEES) ||
				changeType.isType(EventChangeType.USER_DELEGATED_EVENT_CHANGES) ||
				changeType.isType(EventChangeType.OBM_DELETE_DELEGATED_EVENT) ||
				changeType.isType(EventChangeType.USER_PARTICIPATION_CHANGE)) {
			var notificationConfirmationDialog = new Calendar_Email_Notification_Dialog();
			notificationConfirmationDialog.handleNotification(aNewItem,"updated",aOldItem);
		}
	},

	_registerEventModification: function cOC_registerEventModification(aItem, changeType) {
		if (changeType.isType(EventChangeType.USER_EVENT_MAJOR_CHANGES) ||
				changeType.isType(EventChangeType.USER_EVENT_MINOR_CHANGES_WITH_UPDATED_ATTENDEES) ||
				changeType.isType(EventChangeType.USER_EVENT_MINOR_CHANGES_WITHOUT_UPDATED_ATTENDEES) ||
				changeType.isType(EventChangeType.EXTERNAL_EVENT_CHANGES) ||
				changeType.isType(EventChangeType.USER_DELEGATED_EVENT_CHANGES) ||
				changeType.isType(EventChangeType.ATTENDEE_PARTICIPATION_CHANGE)) {
			this.mEventSynchronizer.notifyItemModified(this, aItem);
		}
		else if (changeType.isType(EventChangeType.USER_PARTICIPATION_CHANGE)) {
			this.mEventSynchronizer.notifyItemParticipationChanged(this, aItem);
		}
	},

	_stripAndReturnContextProperties: function cOC_stripAndReturnContextProperties(aItem) {
		var contextProperties = {};
		for each (var contextPropertyName in this.eventContextProps) {
			contextProperties[contextPropertyName] = aItem.getProperty(contextPropertyName);
			// Remove the property to make sure we don't serialize it
			aItem.deleteProperty(contextPropertyName);
		}
		if (aItem.recurrenceInfo != null) {
			for each (var exceptionId in aItem.recurrenceInfo.getExceptionIds({})) {
				this._stripAndReturnContextProperties(aItem.recurrenceInfo.getOccurrenceFor(exceptionId));
			}
		}
		// save the DtStamp... http://ci-obm.linagora.com/fisheye/browse/lightning/calendar-js/calItemBase.js?hb=true#to424
		aItem.setProperty("LAST-MODIFIED", aItem.getProperty("LAST-MODIFIED"));
		return contextProperties;
	},

	// void deleteItem( in calIItemBase aItem, in calIOperationListener aListener );
	deleteItem: function(aItem, aListener) {
		this.logger.debug("deleteItem (" + this.mOBMName + ")", false, false);
		if (this.readOnly) {
			throw Ci.calIErrors.CAL_IS_READONLY;
		}
		this.mLocalCalendar.deleteItem(aItem, aListener);
	},

	// void getItem( in string aId, in calIOperationListener aListener );
	getItem: function(aId, aListener) {
      var self = this;
      var listener = {
        onOperationComplete: function(aCalendar, aStatus, aOperationType, aId, aDetail) {
            aListener.onOperationComplete(self, aStatus, aOperationType, aId, aDetail);
        },
        onGetResult: function(aCalendar, aStatus, aItemType, aDetail, aCount, aItems) {
          aListener.onGetResult(self, aStatus, aItemType, aDetail, aCount, aItems );
          }
      };
    
      try {
        return this.mLocalCalendar.getItem(aId, listener);
      } catch(e) {
        this.logger.error(e, true, false);
      }
	},

	// void getItems( in unsigned long aItemFilter, in unsigned long aCount,
	//                in calIDateTime aRangeStart, in calIDateTime aRangeEndEx,
	//                in calIOperationListener aListener );
	getItems: function(aItemFilter, aCount, aRangeStart, aRangeEnd, aListener) {
		let self = this;
        let listener = {
            onOperationComplete: function (aCalendar, aStatus,aOperationType, aId, aDetail) {
                aListener.onOperationComplete.apply(aListener, arguments); 
            },
            onGetResult: function (aCalendar, aStatus, aItemType, aDetail, aCount, aItems) {
                aListener.onGetResult(self, aStatus, aItemType, aDetail, aCount, aItems);
            }
        };
        try {
          return this.mLocalCalendar.getItems(aItemFilter, aCount, aRangeStart, aRangeEnd, listener);
		} catch(e) {
          this.logger.error(e, true, false);
		}
	},

	// void refresh();
	refresh: function() {
        // Nothing to do, will be taken care of by sync manager
	},

	// void startBatch();
	startBatch: function() {
		//this.logger.debug(this.mOBMName +" -> start batch readOnly:" + this.mLocalCalendar.readOnly, false, false);
		this.synchronizing = true;
		//this.mLocalCalendar.startBatch();
		this.mReadOnlyStateBeforeBatch = this.mLocalCalendar.readOnly;
		this.mLocalCalendar.removeObserver(this.mObserver);
		this.mLocalCalendar.readOnly = false;
	},

	// void endBatch();
	endBatch: function() {
		if (this.synchronizing === true) {
			this.synchronizing = false;
			this.mLocalCalendar.readOnly = this.mReadOnlyStateBeforeBatch;
			//this.mLocalCalendar.endBatch();
			this.mLocalCalendar.addObserver(this.mObserver);
		}
		//this.logger.debug(this.mOBMName +" -> end batch readOnly:" + this.mLocalCalendar.readOnly, false, false);
	},

	//
	// calICalendarProvider interface
	//
	get prefChromeOverlay() {
		return null;
	},

	get displayName() {
		return "Remote OBM Calendar";
	},

	createCalendar: function() {
		throw NS_ERROR_NOT_IMPLEMENTED;
	},

	deleteCalendar: function(aCalendar, aListener) {
		try {
			this.mLocalCalendar.QueryInterface(Ci.calICalendarProvider).deleteCalendar(this.mLocalCalendar, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	//
	// calISchedulingSupport
	//

   isInvitation: function (aItem) {
        let id = this.getProperty("organizerId");
        if (id) {
            let org = aItem.organizer;
            if (!org || (org.id.toLowerCase() == id.toLowerCase())) {
                return false;
            }
            return (aItem.getAttendeeById(id) != null);
        }
        return false;
    },

    hasAttendeesOtherThanOrganizer : function (aItem) {
    	var attendees = aItem.getAttendees({});
    	if (attendees) {
    		var isOrganizerAttendee = this.getInvitedAttendee(aItem);
    		var attendeeCountMinusOrganizer = isOrganizerAttendee ? attendees.length - 1 : attendees.length;
    		return attendeeCountMinusOrganizer >= 1;
    	} else {
    		return false;
    	}
    },

    getInvitedAttendee: function (aItem) {
        let id = this.getProperty("organizerId");
        return (id ? aItem.getAttendeeById(id) : null);
    },

    canNotify: function (aMethod, aItem) {
        return true; // The OBM server sends all the attendee notifications to itself
    },
	//
	// PRIVATE METHODS
	//
	initLocalCalendar: function(aUri) {
		try {
			this.logger.debug("initLocalCalendar()", false, false);
			this.logger.debug("aUri: "+aUri, false, false);
			this.mLocalCalendar = Cc["@mozilla.org/calendar/calendar;1?type=storage"].createInstance(Ci.calICalendar);
			this.mLocalCalendar.uri = aUri;
			this.mLocalCalendar.superCalendar = this;
			this.mLocalCalendar.setProperty("relaxedMode", true);
			this.mObserver = new calOBMObserver(this, this.mOBMName, this.mEventSynchronizer, this.mTaskSynchronizer);
			this.mLocalCalendar.addObserver(this.mObserver);
			this.mLocalCalendar.readOnly = UtilsOBM.getPrefSafe("extensions.obm.changes.calICalendar." + this.mOBMName + ".readOnly", false);

            // This can be removed if the code around
            // capabilities.propagate-sequence goes away in Lightning
            this.mLocalCalendar.setProperty("capabilities.propagate-sequence", false);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	}

};

function calOBMObserver(aOBMCalendar, aOBMName, aEventSynchronizer, aTaskSynchronizer) {
	//Init logger
	this.logger = Cc["@obm.org/calendar/logger;1"].createInstance(Ci.obmILogger);
	var writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Ci.obmILoggerWriter);
	this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
	this.logger.setHeader("calOBMObserver: ");
	this.logger.debug("calOBMObserver()", false, false);

	this.mOBMCalendar = aOBMCalendar;
	this.mOBMName = aOBMName;
	this.mEventSynchronizer = aEventSynchronizer;
	this.mTaskSynchronizer = aTaskSynchronizer;
}

calOBMObserver.prototype = {

	//
	// calIObserver interface
	//
	onStartBatch: function() {
		this.logger.debug("onStartBatch", false, false);
	},

	onEndBatch: function() {
		this.logger.debug("onEndBatch", false, false);
	},

	onLoad: function() {
		this.logger.debug("onLoad", false, false);
	},

	onAddItem: function(aAddedItem) {
		this.logger.debug("onAddItem", false, false);

		if (aAddedItem instanceof Ci.calIEvent) {
			// let's check if this is a new internal OBM event on which
			// only participation has been changed when
			// the user "accept" or "decline" a participation from
			// the mail sent for the participation
			// for this, we check the flag that it indicates we are not
			// in the case were the event is adopted during the synchronisation
			if (aAddedItem.hasProperty('X-OBMEXT-PARTICIPATION-CHANGED')) {
				this.logger.debug("  the added event should be only modified as participation changed on remote server", false, false);
				var notif = new Calendar_Email_Notification_Dialog();
				notif.handleNotification(aAddedItem,"added");
				
				this.mEventSynchronizer.notifyItemParticipationChanged(this.mOBMCalendar, aAddedItem);
			}
			else {
				var notif = new Calendar_Email_Notification_Dialog();
				notif.handleNotification(aAddedItem,"added");

				this.mEventSynchronizer.notifyItemAdded(this.mOBMCalendar, aAddedItem);
			}
			this.logger.debug("aItem id: "+aAddedItem.id,false,false);
			this.logger.debug("aItem uid: "+aAddedItem.uid,false,false);
			this.logger.debug("aItem extId: "+aAddedItem.extId,false,false);
		} else if (aAddedItem instanceof Ci.calITodo) {
			this.mTaskSynchronizer.notifyItemAdded(this.mOBMCalendar, aAddedItem);
		}
	},

	onModifyItem: function(aNewItem, aOldItem) {
	},

	onDeleteItem: function(aDeletedItem) {
		this.logger.debug("onDeleteItem", false, false);

		if (aDeletedItem instanceof Ci.calIEvent) {
            if ( !aDeletedItem.hasProperty("METHOD") && (this.mOBMCalendar.isInvitation(aDeletedItem) || this.mOBMCalendar.hasAttendeesOtherThanOrganizer(aDeletedItem) || UtilsOBM.isEventDelegation(aDeletedItem)) ) {
              var notif = new Calendar_Email_Notification_Dialog();
              notif.handleNotification(aDeletedItem,"removed");
            }
			this.mEventSynchronizer.notifyItemDeleted(this.mOBMCalendar, aDeletedItem);
		} else if (aDeletedItem instanceof Ci.calITodo) {
			this.mTaskSynchronizer.notifyItemDeleted(this.mOBMCalendar, aDeletedItem);
		}
	},

	onAlarm: function(aAlarmItem) {
		this.logger.debug("onAlarm", false, false);
	},

	onError: function(aCalendar, aErrNo, aMessage) {
		this.logger.debug("onError", false, false);
		this.logger.debug("onError: aErrNo: " + aErrNo + " aMessage: " + aMessage, false, false);
	},

	onPropertyChanged: function(aCalendar, aName, aValue, aOldValue) {
		//this.mEventSynchronizer.notifyItemParticipationChanged(this.mOBMCalendar, aNewItem);
		this.logger.debug("onPropertyChanged (" + "aName: " + aName + ", aValue: " + aValue + ", aOldValue: " + aOldValue + ")", false, false);
	},

	onPropertyDeleting: function(aCalendar, aName) {
		this.logger.debug("onPropertyDeleting (aName: " + aName + ")", false, false);
	}
};


function EventChangeType(changeTypes) {
	this.mTypes = {};
	for each (var changeType in changeTypes) {
		this.mTypes[changeType] = true;
	}
}

EventChangeType.USER_EVENT_MAJOR_CHANGES = "USER_EVENT_MAJOR_CHANGES";

EventChangeType.USER_EVENT_MINOR_CHANGES_WITH_UPDATED_ATTENDEES = "USER_EVENT_MINOR_CHANGES_WITH_UPDATED_ATTENDEES";

EventChangeType.USER_EVENT_MINOR_CHANGES_WITHOUT_UPDATED_ATTENDEES = "USER_EVENT_MINOR_CHANGES_WITHOUT_UPDATED_ATTENDEES";

EventChangeType.USER_DELEGATED_EVENT_CHANGES = "USER_DELEGATED_EVENT_CHANGES";

EventChangeType.USER_PARTICIPATION_CHANGE = "USER_PARTICIPATION_CHANGE";

EventChangeType.OBM_DELETE_EVENT = "OBM_DELETE_EVENT";

EventChangeType.OBM_DELETE_DELEGATED_EVENT = "OBM_DELETE_DELEGATED_EVENT";

EventChangeType.ATTENDEE_PARTICIPATION_CHANGE = "ATTENDEE_PARTICIPATION_CHANGE";

EventChangeType.EXTERNAL_EVENT_CHANGES = "EXTERNAL_EVENT_CHANGES";

EventChangeType.prototype = {

	isType: function EventChangeType_isType(changeType) {
		var found = this.mTypes[changeType];
		return (found != null);
	},

	get types() {
        var types = [];
        for (var changeType in this.mTypes) {
            types.push(changeType);
        }
        return types;
	},

}

function EventChanges(aNewItem, aOldItem, updateContext, logger) {
	this.mNewItem = aNewItem;
	this.mOldItem = aOldItem;
	this.mUpdateContext = updateContext;
	this.logger = logger;
	this.updatedAttendees = [];
}

EventChanges.prototype = {
	computeDiffs: function() {
		return new EventComparator(this.mOldItem, this.mNewItem).compare();
	},
	compute: function EventChanges_compute() {
		this.logger.debug("Comparing new item " + this.mNewItem.icalString + " to old item " + this.mOldItem.icalString, false, false);
		var fieldsDiff = this.computeDiffs();
		this.logger.debug("Got field diffs: " + fieldsDiff.eventFields.join(", "))
		if (fieldsDiff.hasNoChanges() || fieldsDiff.hasOnlyAlarmChanges()) {
			this.logger.debug("No change or the changes only concern alarms, not taking them into account");
			return new EventChangeType([]);
		}
		this.logger.debug("Checking if the user is the organizer", false, false);
		var isOrganizer = this.isOrganizer(this.mNewItem);

		var changeTypes = [];

		if (this.mUpdateContext.isFromICS()) {
			this.logger.debug("The event " + this.mNewItem.id + " is from an ICS",
					false, false);
			var isAttendeeParticipationChange = isOrganizer;
			if (isAttendeeParticipationChange) {
				this.logger.debug("The event " + this.mNewItem.id + " is organized by " +
							"the user", false, false);
				changeTypes = [EventChangeType.ATTENDEE_PARTICIPATION_CHANGE];
			}
			else {
				// The organizer must have changed something important, and the
				// user must approved or declined
				this.logger.debug("The event " + this.mNewItem.id + " is not organized " +
						"by the user", false, false);
				if (this.mUpdateContext.method == "CANCEL") {
                  if ( UtilsOBM.isEventDelegation(this.mNewItem) ) {
                    changeTypes = [EventChangeType.OBM_DELETE_DELEGATED_EVENT];
                  } else {
					changeTypes = [EventChangeType.OBM_DELETE_EVENT];
                  }
				}
				else {
					changeTypes = [EventChangeType.USER_PARTICIPATION_CHANGE];
				}
				if (!this.mUpdateContext.isFromSameDomainICS()) {
					this.logger.debug("The update for the event " + this.mNewItem.id +
							" comes from the outside, the changes will be uploaded to OBM",
							false, false);
					changeTypes.push(EventChangeType.EXTERNAL_EVENT_CHANGES);
				}
			}
		}
		else if (this.mUpdateContext.isFromLightning()) {
			this.logger.debug("The event " + this.mNewItem.id + " is from lightning",
					false, false);
            
			if (isOrganizer) {
				this.logger.debug("The event " + this.mNewItem.id + " is organized by " +
							"the user", false, false);
                if ( UtilsOBM.isEventDelegation(this.mNewItem) ) {
                  changeTypes = [EventChangeType.USER_DELEGATED_EVENT_CHANGES];
                } else if (this.hasMajorChanges()) {
					changeTypes = [EventChangeType.USER_EVENT_MAJOR_CHANGES];
				}
				else if (this.hasUpdatedAttendees(this.mNewItem, this.mOldItem)) {
					changeTypes = [EventChangeType.USER_EVENT_MINOR_CHANGES_WITH_UPDATED_ATTENDEES];
				}
				else {
					changeTypes = [EventChangeType.USER_EVENT_MINOR_CHANGES_WITHOUT_UPDATED_ATTENDEES];
				}
			}
			else {
				this.logger.debug("The event " + this.mNewItem.id + " is not organized by " +
							"the user", false, false);
				
				// If the organizer changed consider the change as Major
				if (this.mNewItem.organizer.id == this.mOldItem.organizer.id) {
					this.assertHasChangeInUserAttendeeParticipation();
					changeTypes = [EventChangeType.USER_PARTICIPATION_CHANGE];
				} else {
					changeTypes = [EventChangeType.USER_EVENT_MAJOR_CHANGES];
				}
			}
		}
		else {
			throw new Error("Unknown update context: " + this.mUpdateContext.type);
		}
		return new EventChangeType(changeTypes);
	},

	assertHasChangeInOtherAttendeesParticipation: function EventChanges_assertHasChangeInOtherAttendeesParticipation() {
		if (!this.hasChangeInOtherAttendeesParticipation()) {
			throw new Error("Expected changes in the participation of other " +
					"attendees for new item " + this.mNewItem.icalString +
					" (compared to " + this.mOldItem.icalString + ")");
		}
	},

	assertHasChangeInUserAttendeeParticipation: function EventChanges_assertHasChangeInUserAttendeeParticipation() {
		if (!this.hasChangeInUserAttendeeParticipation()) {
			throw new Error("Expected user participation to have changed in new item " +
					this.mNewItem.icalString + " (compared to " + this.mOldItem.icalString + ")");
		}
	},
    
	hasMajorChanges: function EventChanges_hasMajorChanges() {
		return this.mNewItem.getProperty('SEQUENCE') != this.mOldItem.getProperty('SEQUENCE');
	},

	hasChangeInOtherAttendeesParticipation: function EventChanges_hasChangeInOtherAttendeesParticipation() {
		var self = this;
		var filterCallback = function EventChanges_hasChangeInOtherAttendeesParticipation_filterCallback(attendee) {
			var ownerId = UtilsOBM.calendarOwnMailTo(self.mNewItem);
			return ownerId != attendee.id;
		}
		return this.hasChangeInAttendeesParticipation(this.mNewItem, this.mOldItem, filterCallback);
	},

	hasChangeInUserAttendeeParticipation: function EventChanges_hasChangeInUserAttendeeParticipation() {
		var self = this;
		var filterCallback = function EventChanges_hasChangeInUserAttendeeParticipation_filterCallback(attendee) {
			var ownerId = UtilsOBM.calendarOwnMailTo(self.mNewItem);
			return ownerId == attendee.id;
		}
		return this.hasChangeInAttendeesParticipation(this.mNewItem, this.mOldItem, filterCallback);
	},

	hasChangeInAttendeesParticipation: function EventChanges_hasChangeInAttendeesParticipation(aNewItem, aOldItem, filterCallback) {
		for each (var newAttendee in aNewItem.getAttendees({})) {
			if (filterCallback != null && !filterCallback(newAttendee)) {
				continue;
			}
			var oldAttendee = aOldItem.getAttendeeById(newAttendee.id);
			if (oldAttendee != null &&
					oldAttendee.participationStatus != newAttendee.participationStatus) {
				return true;
			}
			// Don't do anything if oldAttendee is null, maybe we got a reply to
			// a reallly old version of the event
		}
		if (aNewItem.recurrenceInfo != null && aOldItem.recurrenceInfo != null) {
			for each (var newExceptionId in aNewItem.recurrenceInfo.getExceptionIds({})) {
				var newException = aNewItem.recurrenceInfo.getOccurrenceFor(newExceptionId);
				var oldException = aOldItem.recurrenceInfo.getOccurrenceFor(newExceptionId);
				if (oldException != null) {
					if (this.hasChangeInAttendeesParticipation(newException, oldException, filterCallback)) {
						return true;
					}
				}
			}
		}
		return false;
	},

	hasUpdatedAttendees: function EventChanges_hasUpdatedAttendees(aNewItem, aOldItem) {
		var oldAttendees = aOldItem.getAttendees({});
		var newAttendees = aNewItem.getAttendees({});
		if (oldAttendees.length != newAttendees.length) {
			return true;
		}

		for each (var newAttendee in newAttendees) {
			var oldAttendee = aOldItem.getAttendeeById(newAttendee.id);
			if (!oldAttendee) {
				return true;
			}
		}
		if (aNewItem.recurrenceInfo != null && aOldItem.recurrenceInfo != null) {
			for each (var newExceptionId in aNewItem.recurrenceInfo.getExceptionIds({})) {
				var newException = aNewItem.recurrenceInfo.getOccurrenceFor(newExceptionId);
				var oldException = aOldItem.recurrenceInfo.getOccurrenceFor(newExceptionId);
				if (oldException != null) {
					if (this.hasUpdatedAttendees(newException, oldException)) {
						return true;
					}
				}
			}
		}
		return false;
	},

	isOrganizer: function EventChanges_isOrganizer(aItem) {
		var ownerId = UtilsOBM.calendarOwnMailTo(aItem);
		var ownerName = cal.getCalendarManager().getCalendarPref_(aItem.calendar, "X-OBM-EMAIL");
		var eventOrganizer = aItem.organizer;
		var isOrganizer;
		if (!eventOrganizer) {
			throw new Error("No organizer for event " + aItem.icalString);
		}
		else {
			isOrganizer = (eventOrganizer.id == ownerId);
		}
		return isOrganizer;
	},
}

function UpdateContext(contextProperties) {
	var type = null;
	if (contextProperties["X-OBM-ORIGIN"] == "obm-sync") {
		type = UpdateContext.OBM_SYNC;
	}
	else if (contextProperties["METHOD"] == null) {
		type = UpdateContext.LIGHTNING;
	}
	else if (contextProperties['X-OBM-DOMAIN-UUID'] != null &&
			contextProperties['X-OBM-DOMAIN-UUID'] != UtilsOBM.domainUuid.get()) {
		type = UpdateContext.OTHER_OBM_DOMAIN_ICS;
	}
	else if (contextProperties['X-OBM-DOMAIN'] == null && contextProperties['X-OBMEXT-IS-INTERNAL'] == null) {
		type = UpdateContext.OTHER_NON_OBM_DOMAIN_ICS;
	}
	else {
		type = UpdateContext.SAME_OBM_DOMAIN_ICS;
	}
	this.type = type;
	this.method = contextProperties["METHOD"];
}

UpdateContext.OBM_SYNC = "OBM_SYNC";
UpdateContext.LIGHTNING = "LIGHTNING";
UpdateContext.OTHER_OBM_DOMAIN_ICS = "OTHER_OBM_DOMAIN_ICS";
UpdateContext.OTHER_NON_OBM_DOMAIN_ICS = "OTHER_NON_OBM_DOMAIN_ICS";
UpdateContext.SAME_OBM_DOMAIN_ICS = "SAME_OBM_DOMAIN_ICS";

UpdateContext.prototype = {
	isFromICS: function UpdateContext_isFromICS() {
		return ([UpdateContext.SAME_OBM_DOMAIN_ICS, UpdateContext.OTHER_OBM_DOMAIN_ICS,
				UpdateContext.OTHER_NON_OBM_DOMAIN_ICS].indexOf(this.type) >= 0);
	},

	isFromSameDomainICS: function UpdateContext_isFromSameDomainICS() {
		return this.type == UpdateContext.SAME_OBM_DOMAIN_ICS;
	},

	isFromLightning: function UpdateContext_isFromLightning() {
		return this.type == UpdateContext.LIGHTNING;
	}
}

/****
 **** module registration
 ****/

var components = [calOBMCalendar];
if ("generateNSGetFactory" in XPCOMUtils) {
    var NSGetFactory = XPCOMUtils.generateNSGetFactory(components);
} else {
    var NSGetModule = XPCOMUtils.generateNSGetModule(components);
}
