/******************************************************************************
Copyright (C) 2011-2012 Linagora

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version, provided you comply with the Additional Terms applicable for OBM
connector by Linagora pursuant to Section 7 of the GNU Affero General Public
License, subsections (b), (c), and (e), pursuant to which you must notably (i)
retain the “Message sent thanks to OBM, Free Communication by Linagora”
signature notice appended to any and all outbound messages (notably e-mail and
meeting requests), (ii) retain all hypertext links between OBM and obm.org, as
well as between Linagora and linagora.com, and (iii) refrain from infringing
Linagora intellectual property rights over its trademarks and commercial
brands. Other Additional Terms apply, see <http://www.linagora.com/licenses/>
for more details.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License and
its applicable Additional Terms for OBM along with this program. If not, see
<http://www.gnu.org/licenses/> for the GNU Affero General   Public License
version 3 and <http://www.linagora.com/licenses/> for the Additional Terms
applicable to OBM connectors.
******************************************************************************/

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

//
let _obmTypesMappings = {};
_obmTypesMappings["CELL;VOICE;X-OBM-Ref1"] = "mobilePhone";
_obmTypesMappings["PAGER;X-OBM-Ref1"] = "pagerPhone";
_obmTypesMappings["WORK;VOICE;X-OBM-Ref1"] = "workPhone";
_obmTypesMappings["WORK;FAX;X-OBM-Ref1"] = "workFax";
_obmTypesMappings["HOME;VOICE;X-OBM-Ref1"] = "homePhone";
_obmTypesMappings["INTERNET;X-OBM-Ref1"] = "email";
_obmTypesMappings["INTERNET;X-OBM-Ref2"] = "email2";
_obmTypesMappings["URL;X-OBM-Ref1"] = "workWebPage";
_obmTypesMappings["URL;X-OBM-Ref2"] = "homeWebPage";
_obmTypesMappings["HOME;X-OBM-Ref1"] = "home";
_obmTypesMappings["WORK;X-OBM-Ref1"] = "work";

//
let _contactTypesMappings = {};
_contactTypesMappings["mobilePhone"] = "CELL;VOICE;X-OBM-Ref1";
_contactTypesMappings["pagerPhone"] = "PAGER;X-OBM-Ref1";
_contactTypesMappings["workPhone"] = "WORK;VOICE;X-OBM-Ref1";
_contactTypesMappings["workFax"] = "WORK;FAX;X-OBM-Ref1";
_contactTypesMappings["homePhone"] = "HOME;VOICE;X-OBM-Ref1";
_contactTypesMappings["email"] = "INTERNET;X-OBM-Ref1";
_contactTypesMappings["email2"] = "INTERNET;X-OBM-Ref2";
_contactTypesMappings["workWebPage"] = "URL;X-OBM-Ref1";
_contactTypesMappings["homeWebPage"] = "URL;X-OBM-Ref2";
_contactTypesMappings["home"] = "HOME;X-OBM-Ref1";
_contactTypesMappings["work"] = "WORK;X-OBM-Ref1";

function _xS(value) {
    return (value ? "" + value : "");
}

function _xL(value, def) {
    if (typeof value == "number" ) {
        return value;
    } else if (typeof value == "string") {
        return parseInt(value, 10);
    } else if (def) {
        return def;
    } else {
        return 0;
    }
}

function _sF(value) {
    return (value ? value : null);
}

function serializeRContact(aContact /* obmIRContact */) {
    // xmlns="http://www.obm.org/xsd/sync/contact.xsd"/>;
    let doc = OBMXMLCreateDoc("contact");
    let docEl = doc.documentElement;

    function _appendXMLElement(aName, aValue, aParent) {
      if (aValue) {
          let parent = aParent || docEl;
          let elem = doc.createElement(aName);
          parent.appendChild(elem);
          elem.textContent = aValue;
          return elem;
      }
    }

    if (aContact.uid) {
        docEl.setAttribute("uid", aContact.uid);
    }

    _appendXMLElement("first", aContact.firstName);
    _appendXMLElement("last", aContact.lastName);
    _appendXMLElement("aka", aContact.aka);
    _appendXMLElement("commonname", aContact.commonname);
    _appendXMLElement("title", aContact.title);
    _appendXMLElement("service", aContact.service);
    _appendXMLElement("company", aContact.company);
    // FIXME is read-only ?
    _appendXMLElement("comment", aContact.comment);
    _appendXMLElement("caluri", aContact.caluri);

    // emails
    let emails = docEl.appendChild(doc.createElement("emails"));
    for each (let key in ["email", "email2"]) {
        if (aContact[key]) {
            let mail = emails.appendChild(doc.createElement("mail"));
            mail.setAttribute("label", _contactTypesMappings[key]);
            mail.setAttribute("value", aContact[key]);
        }
    }

    // phones
    let phones = docEl.appendChild(doc.createElement("phones"));
    let phoneKeys = ["workPhone", "homePhone", "workFax",
                     "pagerPhone", "mobilePhone"];
    for each (let key in phoneKeys) {
        if (aContact[key]) {
            let phone = phones.appendChild(doc.createElement("phone"));
            phone.setAttribute("label", _contactTypesMappings[key]);
            phone.setAttribute("number", aContact[key]);
        }
    }

    // home and work addresses and websites
    let addrs = docEl.appendChild(doc.createElement("addresses"));
    let websites = docEl.appendChild(doc.createElement("websites"));
    for each (let key in ["home", "work"]) {
        if (aContact[key + "Street"] || aContact[key + "Country"] ||
            aContact[key + "City"] || aContact[key + "State"] ||
            aContact[key + "ZipCode"]) {

            // Add an address element for the current address
            let addr = addrs.appendChild(doc.createElement("address"));
            addr.textContent = _xS(aContact[key + "Street"]);
            addr.setAttribute("label", _contactTypesMappings[key]);
            addr.setAttribute("country", _xS(aContact[key + "Country"]));
            addr.setAttribute("town", _xS(aContact[key + "City"]));
            addr.setAttribute("state", _xS(aContact[key + "State"]));
            addr.setAttribute("zip", _xS(aContact[key + "ZipCode"]))

            // Now the websites
            if (aContact[key + "WebPage"]) {
                let site = websites.appendChild(doc.createElement("site"));
                site.setAttribute("label", _contactTypesMappings[key + "WebPage"]);
                site.setAttribute("url", aContact[key + "WebPage"]);
            }
        }
    }

    // TODO no child elements. really needed?
    let instantmessaging = docEl.appendChild(doc.createElement("instantmessaging"));
    return '<?xml version="1.0" encoding="UTF-8"?>' + OBMSerializeDOM(doc);
}

function getBookName(aBook) {
    //crappy !!!
    if (aBook == 0)
        return "users";
    return "contacts";
}

function ObmRAddressBookProxy() {
    try {
        Components.utils.import("resource://obm-extension/modules/UtilsOBM.jsm");
        Components.utils.import("resource://obm-extension/modules/XMLUtilsOBM.jsm");
    } catch (e) {
        Components.utils.reportError(e);
        dump(e + "\n");
    }

    //Init logger
    this.logger = Cc["@obm.org/calendar/logger;1"].createInstance(Ci.obmILogger);
    var writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Ci.obmILoggerWriter);
    this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
    this.logger.setHeader("obmAddressBookProxy: ");
    this.logger.debug("new obmAddressBookProxy()", false, false);

    this.RESTClient = Cc["@obm.org/restclient;1"].getService().wrappedJSObject;

    this.mListeners = [];
    this.mUrl = null;
    this.mLoginUrl = null;
    this.mToken = {
        QueryInterface: XPCOMUtils.generateQI([Ci.obmIRAuthToken]),
        session_id: null
    };
};

ObmRAddressBookProxy.prototype = {
    QueryInterface: XPCOMUtils.generateQI([Ci.obmIRAddressBookProxy, Ci.obmIRProxyBase]),
    classID: Components.ID("{4b4d13f2-53c0-439a-85c9-f9ac654f72c7}"),
    contractID: "@obm.org/obm-sync-rest/addressbookproxy;1",
    classDescription: "OBM Address Book Proxy",

    /*
     * obmIRProxyBase
     */
    get token() {
        return this.mToken;
    },
    set token(aToken) {
        this.mToken = aToken;
    },

    get PortUrl() {
        return this.mUrl;
    },
    set PortUrl(aURL) {
        if(aURL.charAt(aURL.length -1) == '/') {
          aURL = aURL.substring(0, aURL.length - 1);
        }
        this.mUrl = aURL + "/book";
        this.mLoginUrl = aURL + "/login";
    },

    addListener: function(listener) {
        for (var i = 0; i < this.mListeners.length; ++i) {
            if (this.mListeners[i] == listener)
                return;
        }
        this.mListeners.push(listener);
    },

    removeListener: function(listener) {
        this.mListeners = [];
    },

    _notifyListeners: function(aResultCode, aMethodName, aObj, aData) {
        // A copy is needed, a new listeners could has been added during the callbacks
        var listeners = this.mListeners.slice(0, this.mListeners.length);
        for (var i = 0; i < listeners.length; ++i) {
            try {
                //timer.(function() {
                listeners[i].onResponse(aResultCode, aMethodName, aObj, aData);
                //}, 1, timer.TYPE_ONE_SHOT);
            } catch (e) {
                //Continue to notify listeners
                this.logger.error(e + " (ignored)");
            }
        }
    },

    /*
     * obmIRAddressBookProxy
     */
    _callback: function(aMethodName, aObj, aDoc, aData) {
        this.logger.debug("_callback(aMethodName = " + aMethodName + ")", false, false);

        let docEl = aDoc && aDoc.documentElement;

        if (aObj && aObj.name == "Error") {
            var msg = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);

            if (aMethodName == "doLogin" &&  aObj.fileName != "Connection failure" && aObj.fileName != "Server internal error") {
              if (UtilsOBM.capabilitySupported("LOGIN_HANDLER_SUPPORTS_AUTHFAULT")) {
                let exception = OBMXPathFirst(docEl, "type/text()");

                if (exception == "org.obm.sync.auth.AuthFault") {
                  msg.data = "No token available";
                } else {
                  msg.data = aObj.fileName;
                }
              } else {
                msg.data = "No token available";
              }
            } else {
                let exception = OBMXPathFirst(docEl, "type/text()")

                if (exception == "javax.naming.NoPermissionException") {
                    msg.data = "NotAllowed";
                } else {
                    msg.data = aObj.fileName;
                }
            }

            if (aObj.fileName == "Invalid Certificate") {
              msg.data = "Invalid Certificate";
            }

            this.logger.debug("_callback(Error = " + aObj.message + " " + aObj.fileName + ")", false, false);
            this._notifyListeners(Cr.NS_ERROR_FAILURE, aMethodName, msg, aData);

            return;
        }

        switch(aMethodName) {
            case "doLogin":
                let sid = OBMXPathFirst(docEl, "sid/text()");
                this.mToken.session_id = sid;
                this.logger.debug("token obtained with sid:"+sid, false, false);
                UtilsOBM.domainUuid.getFromLoginResponse(docEl);
                this._notifyListeners(Cr.NS_OK, aMethodName, null, aData);
                break;
            case "getSync":
            case "listAllChanges":
                this._notifyListeners(Cr.NS_OK, aMethodName, new GetSyncContactResponseShell(docEl), aData);
                break;
            case "getAddressBookSync":
                this._notifyListeners(Cr.NS_OK, aMethodName, new AddressBookChangesResponseShell(docEl), aData);
                break;
            case "getContactFromId":
            case "createContact":
            case "createContactInBook":
            case "storeContact":
            case "createContactWithoutDuplicate":
            case "modifyContact":
            case "removeContact":
                this._notifyListeners(Cr.NS_OK, aMethodName, new ContactShell(docEl), aData);
                break;
            case "unsubscribeBook":
                /* TODO: handle boolean return */
                this._notifyListeners(Cr.NS_OK, aMethodName, null, aData);
                break;
            default:
                this._notifyListeners(Cr.NS_ERROR_FAILURE, aMethodName, null, aData);
                break;
        }
    },

    getSync: function(aBook, aLastSync, aData) {
        this.logger.debug("getSync()", false, false);
        var p = this.RESTClient.newRESTClientParameters();
        p.add("sid", this.mToken.session_id);
        p.add("book", getBookName(aBook));
        p.add("lastSync", aLastSync);
        // closure
        var self = this;
        this.RESTClient.invoke(this.mUrl, "getSync", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
    },
    getAddressBookSync: function(aLastSync, aData) {
        this.logger.debug("getAddressBookSync", false, false);
        var p = this.RESTClient.newRESTClientParameters();
        p.add("sid", this.mToken.session_id);
        p.add("lastSync", aLastSync);
        // closure
        var self = this;
        this.RESTClient.invoke(this.mUrl, "getAddressBookSync", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
    },
    listAllChanges: function(aLastSync, aBookId, aData) {
        this.logger.debug("listAllChanges(lastSync=" + aLastSync + ", aBookId=" + aBookId + ")", false, false);
        var p = this.RESTClient.newRESTClientParameters();
        p.add("sid", this.mToken.session_id);
        p.add("lastSync", aLastSync);
        p.add("bookId", aBookId);
        // closure
        var self = this;
        this.RESTClient.invoke(this.mUrl, "listAllChanges", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
    },
    _createContactInternal: function(aBook, aContact, aUniq, aData) {
        var p = this.RESTClient.newRESTClientParameters();
        p.add("sid", this.mToken.session_id);
        p.add("book", getBookName(aBook));
        p.add("contact", serializeRContact(aContact));
        // closure
        var self = this;
        this.RESTClient.invoke(this.mUrl, aUniq ? "createContactWithoutDuplicate" : "createContact", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
    },
    createContact: function(aBook, aContact, aData) {
        this.logger.debug("createContact()", false, false);
        return this._createContactInternal(aBook, aContact, false, aData);
    },
    createContactWithoutDuplicate: function(aBook, aContact, aData) {
        this.logger.debug("createContactWithoutDuplicate()", false, false);
        return this._createContactInternal(aBook, aContact, true, aData);
    },
    createContactInBook: function(aBookId, aContact, aData) {
        var p = this.RESTClient.newRESTClientParameters();
        p.add("sid", this.mToken.session_id);
        p.add("bookId", aBookId);
        p.add("contact", serializeRContact(aContact));
        // closure
        var self = this;
        this.RESTClient.invoke(this.mUrl, "createContactInBook", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
    },
    modifyContact: function(aBook, aContact, aData) {
        this.logger.debug("modifyContact()", false, false);
        var p = this.RESTClient.newRESTClientParameters();
        p.add("sid", this.mToken.session_id);
        p.add("book", getBookName(aBook));
        p.add("contact", serializeRContact(aContact));
        // closure
        var self = this;
        this.RESTClient.invoke(this.mUrl, "modifyContact", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
    },
    storeContact: function(aBookId, aContact, aData) {
        var p = this.RESTClient.newRESTClientParameters();
        p.add("sid", this.mToken.session_id);
        p.add("bookId", aBookId);
        p.add("contact", serializeRContact(aContact));

        var self = this;
        this.RESTClient.invoke(this.mUrl, "storeContact", p, true, function(name, obj, xml, data) {
            self._callback(name, obj, xml, data);
        }, aData);
    },
    removeContact: function(aBook, aUid, aData) {
        this.logger.debug("removeContact()", false, false);
        var p = this.RESTClient.newRESTClientParameters();
        p.add("sid", this.mToken.session_id);
        p.add("book", getBookName(aBook));
        p.add("id", aUid);
        // closure
        var self = this;
        this.RESTClient.invoke(this.mUrl, "removeContact", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
    },
    unsubscribeBook: function(aBookId, aData) {
        var p = this.RESTClient.newRESTClientParameters();
        p.add("sid", this.mToken.session_id);
        p.add("bookId", aBookId);
        // closure
        var self = this;
        this.RESTClient.invoke(this.mUrl, "unsubscribeBook", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
    },
    listBooks: function(aData) {
        throw Cr.NS_ERROR_NOT_IMPLEMENTED;
    },
    getContactFromId: function(aBook, aUid, aData) {
        this.logger.debug("getContactFromId()", false, false);
        var p = this.RESTClient.newRESTClientParameters();
        p.add("sid", this.mToken.session_id);
        p.add("book", getBookName(aBook));
        p.add("id", aUid);
        // closure
        var self = this;
        this.RESTClient.invoke(this.mUrl, "getContactFromId", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
    },
    getContactTwinKeys: function (aBook, aContact, aData) {
        throw Cr.NS_ERROR_NOT_IMPLEMENTED;
    },
    doLogin: function(aLogin, aPwd, aOrigin, aData) {
        this.logger.debug("doLogin()", false, false);
        var p = this.RESTClient.newRESTClientParameters();
        p.add("login", aLogin);
        p.add("password", aPwd);
        p.add("origin", aOrigin);
        // closure
        var self = this;
        this.RESTClient.invoke(this.mLoginUrl, "doLogin", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
    },
    isReadOnly: function(aBook, aData) {
        throw Cr.NS_ERROR_NOT_IMPLEMENTED;
    }
};

function AddressBookShell(aNode) {
    this.mNode = aNode;
}

AddressBookShell.prototype = {
    QueryInterface: XPCOMUtils.generateQI([Ci.obmIRAddressBook]),

    get uid() { return _xL(this.mNode.getAttribute("uid")); },
    get name() { return _xS(OBMXPathFirst(this.mNode, "name/text()")); },
    get ownerDisplayName() { return _xS(OBMXPathFirst(this.mNode, "ownerDisplayName/text()")); }
}

function ContactShell(aNode) {
    this.mNode = aNode;

    this.mPhones = {};
    for each (let phone in OBMXPath(aNode, "phones/*")) {
        let unmappedType = _xS(phone.getAttribute("label")).replace("PREF;","");
        let type = _obmTypesMappings[unmappedType];
        if (type) {
            this.mPhones[type] = _xS(phone.getAttribute("number"));
        }
    }

    this.mEmails = {};
    for each (let mail in OBMXPath(aNode, "emails/*")) {
        let unmappedType = _xS(mail.getAttribute("label")).replace("PREF;","");
        let type = _obmTypesMappings[unmappedType];
        if (type) {
            this.mEmails[type] = _xS(mail.getAttribute("value"));
        }
    }

    this.mWebSites = {};
    for each (let site in OBMXPath(aNode, "websites/*")) {
        let unmappedType = _xS(site.getAttribute("label")).replace("PREF;","");
        let type = _obmTypesMappings[unmappedType];
        if (type) {
            this.mWebSites[type] = _xS(site.getAttribute("url"));
        }
    }

    this.mAddresses = [];
    for each (let address in OBMXPath(aNode, "addresses/*")) {
        let unmappedType = _xS(address.getAttribute("label")).replace("PREF;","");
        let type = _obmTypesMappings[unmappedType];
        if (type) {
            this.mAddresses[type] = {
                street: _xS(address.textContent),
                state: _xS(address.getAttribute("state")),
                city: _xS(address.getAttribute("town")),
                country: _xS(address.getAttribute("country")),
                zipCode: _xS(address.getAttribute("zip")),
                expressPostal: _xS(address.getAttribute("expressPostal"))
            };
        }
    }
}

ContactShell.prototype = {
    QueryInterface: XPCOMUtils.generateQI([Ci.obmIRContact]),

    get uid() { return _xS(this.mNode.getAttribute("uid")); },
    get addressbookid() { return _xL(OBMXPathFirst(this.mNode, "addressbookid/text()")); },
    get firstName() { return _xS(OBMXPathFirst(this.mNode, "first/text()")); },
    get lastName() { return _xS(OBMXPathFirst(this.mNode, "last/text()")); },
    get commonname() { return _xS(OBMXPathFirst(this.mNode, "commonname/text()")); },
    get aka() { return _xS(OBMXPathFirst(this.mNode, "aka/text()")); },
    get email() { return _sF(this.mEmails["email"])},
    get email2() { return _sF(this.mEmails["email2"])},
    get workPhone() { return _sF(this.mPhones["workPhone"])},
    get homePhone() { return _sF(this.mPhones["homePhone"])},
    get workFax() { return _sF(this.mPhones["workFax"])},
    get pagerPhone() { return _sF(this.mPhones["pagerPhone"])},
    get mobilePhone() { return _sF(this.mPhones["mobilePhone"])},
    get homeStreet() { return ((this.mAddresses["home"]) ? this.mAddresses["home"].street : null)},
    get homeZipCode() { return ((this.mAddresses["home"]) ? this.mAddresses["home"].zipCode : null)},
    get homeCity() { return ((this.mAddresses["home"]) ? this.mAddresses["home"].city : null)},
    get homeState() { return ((this.mAddresses["home"]) ? this.mAddresses["home"].state : null)},
    get homeExpressPostal() { return ((this.mAddresses["home"]) ? this.mAddresses["home"].expressPostal : null)},
    get homeCountry() { return ((this.mAddresses["home"]) ? this.mAddresses["home"].country : null)},
    get homeWebPage() { return _sF(this.mWebSites["homeWebPage"])},
    get title() { return _xS(OBMXPathFirst(this.mNode, "title/text()")); },
    get service() { return _xS(OBMXPathFirst(this.mNode, "service/text()")); },
    get company() { return _xS(OBMXPathFirst(this.mNode, "company/text()")); },
    get workStreet() { return ((this.mAddresses["work"]) ? this.mAddresses["work"].street : null)},
    get workZipCode() { return ((this.mAddresses["work"]) ? this.mAddresses["work"].zipCode : null)},
    get workCity() { return ((this.mAddresses["work"]) ? this.mAddresses["work"].city : null)},
    get workState() { return ((this.mAddresses["work"]) ? this.mAddresses["work"].state : null)},
    get workExpressPostal() { return ((this.mAddresses["work"]) ? this.mAddresses["work"].expressPostal :null)},
    get workCountry() { return ((this.mAddresses["work"]) ? this.mAddresses["work"].country : null)},
    get workWebPage() { return _sF(this.mWebSites["workWebPage"])},
    get comment() { return _xS(OBMXPathFirst(this.mNode, "comment/text()")); },
    get caluri()  { return _xS(OBMXPathFirst(this.mNode, "caluri/text()")); }
};

function AddressBookChangesResponseShell(aNode) {
    this.mUpdated = [];
    this.mRemoved = [];
    this.mUpdatedAB = [];
    this.mRemovedAB = [];
    this.mLastSync = aNode.getAttribute("lastSync");

    for each (var updatedAB in OBMXPath(aNode, "addressbooks/updated/*")) {
        this.mUpdatedAB.push(new AddressBookShell(updatedAB));
    }

    for each (var removedAB in OBMXPath(aNode, "addressbooks/removed/*")) {
        this.mRemovedAB.push(removedAB.getAttribute("uid"));
    }

    for each (var updated in OBMXPath(aNode, "contacts/updated/*")) {
        this.mUpdated.push(new ContactShell(updated));
    }

    for each (var removed in OBMXPath(aNode, "contacts/removed/*")) {
        this.mRemoved.push(removed.getAttribute("uid"));
    }
}

AddressBookChangesResponseShell.prototype = {
    QueryInterface: XPCOMUtils.generateQI([Ci.obmIRAddressBookChangesResponse]),

    getUpdatedAddressBooks: function(count) {
        count.value = this.mUpdatedAB.length;
        return this.mUpdatedAB;
    },
    getRemovedAddressBooksIds: function(count) {
        count.value = this.mRemovedAB.length;
        return this.mRemovedAB;
    },
    getUpdatedContacts: function(count) {
        count.value = this.mUpdated.length;
        return this.mUpdated;
    },
    getRemovedContactIds: function(count) {
        count.value = this.mRemoved.length;
        return this.mRemoved;
    },
    get lastsync() {
        return this.mLastSync;
    }
};

function GetSyncContactResponseShell(aNode) {
    this.mUpdated = [];
    this.mRemoved = [];
    this.mLastSync = aNode.getAttribute("lastSync");

    for each (var updated in OBMXPath(aNode, "updated/*")) {
        this.mUpdated.push(new ContactShell(updated));
    }

    for each (var removed in OBMXPath(aNode, "removed/*")) {
        this.mRemoved.push(removed.getAttribute("uid"));
    }
}

GetSyncContactResponseShell.prototype = {
    QueryInterface: XPCOMUtils.generateQI([Ci.obmIRAddressBookSyncResponse]),
    getUpdatedContacts: function(count) {
        count.value = this.mUpdated.length;
        return this.mUpdated;
    },
    getRemovedContactIds: function(count) {
        count.value = this.mRemoved.length;
        return this.mRemoved;
    },
    get lastsync() {
        return this.mLastSync;
    }
};

// module
var components = [ObmRAddressBookProxy];
if ("generateNSGetFactory" in XPCOMUtils) {
    var NSGetFactory = XPCOMUtils.generateNSGetFactory(components);
} else {
    var NSGetModule = XPCOMUtils.generateNSGetModule(components);
}
