/******************************************************************************
Copyright (C) 2011-2012 Linagora

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version, provided you comply with the Additional Terms applicable for OBM
connector by Linagora pursuant to Section 7 of the GNU Affero General Public
License, subsections (b), (c), and (e), pursuant to which you must notably (i)
retain the “Message sent thanks to OBM, Free Communication by Linagora”
signature notice appended to any and all outbound messages (notably e-mail and
meeting requests), (ii) retain all hypertext links between OBM and obm.org, as
well as between Linagora and linagora.com, and (iii) refrain from infringing
Linagora intellectual property rights over its trademarks and commercial
brands. Other Additional Terms apply, see <http://www.linagora.com/licenses/>
for more details.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License and
its applicable Additional Terms for OBM along with this program. If not, see
<http://www.gnu.org/licenses/> for the GNU Affero General   Public License
version 3 and <http://www.linagora.com/licenses/> for the Additional Terms
applicable to OBM connectors.
******************************************************************************/

var EXPORTED_SYMBOLS = ["Calendar_Email_Notification_Dialog"];

Components.utils.import("resource://obm-extension/modules/UtilsOBM.jsm");
Components.utils.import("resource://calendar/modules/calUtils.jsm");
var logger;

// Before the synchro, a new event with no attendee will have no
// attendee (or organizers)
// After the synchro, there will be a single attendee who will be the
// organizer
var otherAttendeesCount = function(event, calendarEmail) {
    var attendees = event.getAttendees({});
    if ( !attendees.length ) { return 0; }
    var calendarOwnerAttendee = event.getAttendeeById("MAILTO:"+calendarEmail);
    if ( calendarOwnerAttendee ) {
        return (attendees.length - 1);
    }
    return attendees.length;
};

var organizerExists = function(event) {
  return event.organizer ? true : false;
};

var isOrganizer = function(event, calendarEmail) {
  if ( !organizerExists(event) || event.organizer.id.replace(/mailto:/ig, "") != calendarEmail.toLowerCase() ) {
    return false;
  }
  return true;
};


var eventOnlyContainsThisAttendee = function(event, calendarEmail) {
  if ( organizerExists(event) && !isOrganizer(event, calendarEmail) ) {
    return false;
  }
  return (otherAttendeesCount(event, calendarEmail) == 0) ;
};

function Calendar_Email_Notification_Dialog () {
    logger = Components.classes["@obm.org/calendar/logger;1"].createInstance(Components.interfaces.obmILogger);
    var writer = Components.classes["@obm.org/calendar/writer;1?type=unixfile"].getService(Components.interfaces.obmILoggerWriter);
    logger.setLevelWriter(logger.LOAD_LEVEL, writer);
    logger.setHeader("Calendar_Email_Notification_Dialog: ");
    logger.debug("new Calendar_Email_Notification_Dialog()", false, false);
}

Calendar_Email_Notification_Dialog.prototype = {
    // item : calendar event item
    // action : "added", "updated", "removed"
    // olditem: in case of action == "updated", the old state of the item
    handleNotification: function(item, action, oldItem) {
        
        if (!cal.isEvent(item)) {
            logger.debug("item is not an event: no notification widget",false,false);
            return;
        }
        
        // don't even think of notifying events older than 6 months
        // Compat with utilsOBM => purge
        var d = new Date();
        var dd = new Date(d.getFullYear(), d.getMonth() - 6, d.getDay(), d.getHours());
        var rangeEnd = cal.jsDateToDateTime(dd, cal.calendarDefaultTimezone());

        if (item.startDate.compare(rangeEnd) < 0) {
            logger.debug("compare event date before rangeEnd",false,false);
            return;
        }
        
        // check the owner of the calendar & the current user
        // if the current user is not the owner of the calendar the notification
        // can happen even if there is only one attendee
        // (case of the secretary taking an appointment for her boss)
        var calendarOwner = UtilsOBM.getCalendarOBMName(item.calendar);
        var tbirdUser = {}, tbirdPasswd = {}, obmLogin, attendees = [];
        var success = UtilsOBM.getOBMCredentials(tbirdUser, tbirdPasswd);
        if ( success ) {
            obmLogin = tbirdUser.value.split("@").shift();
            logger.debug("calendarOwner: "+calendarOwner+", tbird User: "+ obmLogin ,false,false);
        }
        
        if ( !success || obmLogin == calendarOwner ) {
            var calendarEmail = UtilsOBM.calendarOwnerEmail(item);
            var foo = {};
            logger.debug("attendees : "+ item.getAttendees(foo).length,false,false);
            attendees = item.getAttendees(foo) ; 
            if ( eventOnlyContainsThisAttendee(item, calendarEmail) ) {
                // only one attendee, which is the user herself 
                if ( !oldItem ) {
                    logger.debug("No other attendee than the organizer : no notification widget",false,false);
                    return ;
                }
                
                //we should check whether an attendee has been removed (OBMFULL-3228)
                if ( otherAttendeesCount(oldItem, calendarEmail) == 0 ) {
                  logger.debug("No other attendee than the organizer : no notification widget",false,false);
                  return ;
                }
            }
        }
        
        // the case where an event is created via drag, in another users calendar :
        // the notification popup shouldn't show up
        if ( obmLogin != calendarOwner && action == "added" ) {
            if ( UtilsOBM.isDefaultEventTitle(item) && attendees.length == 0 ) {
                logger.debug("Event created in another user's calendar without further modification : no notification widget",false,false);
                return ;
            }
        }
        
        let pref = UtilsOBM.getPrefSafe("extensions.obm.sendNotifications", "ask");
        if ( pref == "always" ) {
            //logger.debug("setting notification to true from user preferences",false,false);
            UtilsOBM.setNotificationStatus(item.calendar, item,action,true);
            return;
        } else if ( pref == "never" ) {
            //logger.debug("setting notification to false from user preferences",false,false);
            UtilsOBM.setNotificationStatus(item.calendar, item,action,false);
            return;
        }
        
        this.openWindow(item,action);
    },
    
    /**
     * Opens a new email notification window asking user if emails should be sent
     * for the current notification
     */
    openWindow: function ( aItem, action ) {
        var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
            .getService(Components.interfaces.nsIWindowWatcher);
        var paramBlock = Components.classes["@mozilla.org/embedcomp/dialogparam;1"]
            .createInstance(Components.interfaces.nsIDialogParamBlock);

        paramBlock.SetNumberStrings(4);
        paramBlock.SetString(0, aItem.calendar.id);
        paramBlock.SetString(1, aItem.id);
        paramBlock.SetString(2, aItem.title);
        paramBlock.SetString(3, action);
        
        var windowName = "notificationDialog:"+aItem.id;
        
        var win = ww.openWindow(null, "chrome://obm-extension/content/email-notifications.xul",
            windowName, "chrome,centerscreen,resizable,modal", paramBlock);
    }
};
