var EXPORTED_SYMBOLS = ["SubscriptionWidget"];

Components.utils.import("resource://obm-extension/modules/paginer.jsm");
Components.utils.import("resource://obm-extension/modules/infiniteTree.jsm");

var UO_logger = Components.classes["@obm.org/calendar/logger;1"].createInstance(Components.interfaces.obmILogger);
var UO_writer = Components.classes["@obm.org/calendar/writer;1?type=unixfile"].getService(Components.interfaces.obmILoggerWriter);
UO_logger.setLevelWriter(UO_logger.LOAD_LEVEL, UO_writer);
UO_logger.setHeader("subscriptionWidget.jsm: ");

function SubscriptionWidget(dom, treeRowBuilder, limit, fetcher, fetcherMethod) {
  this.domElements = {
    error: dom.querySelector('.error'),
    loading: dom.querySelector('.loading'),
    tree: dom.querySelector('tree'),
    treeChildren: dom.querySelector('treechildren'),
    searchField: dom.querySelector('textbox')
  };
  this.treeWidget = new InfiniteTree('foo', this.domElements.tree, this.domElements.treeChildren, this.fetchNextItems.bind(this));
  this.paginer = new Paginer(fetcherMethod, limit, 'init', fetcher);
  this.treeRowBuilder = treeRowBuilder;
  this._bindEvents();
};

SubscriptionWidget.prototype._bindEvents = function() {
  this.domElements.searchField.addEventListener('command', function() {
    this.search();
  }.bind(this), false);
};

SubscriptionWidget.prototype.setTreeObserver = function(MutationObserver, observerCallback) {
  if ( MutationObserver ) {
    var observer = new MutationObserver(observerCallback);
    var config = { attributes: true, childList: false, characterData: false, attributeFilter: ['value'], subtree: true };
    observer.observe(this.domElements.tree, config);  
  } else {
    this.domElements.treeChildren.addEventListener('DOMAttrModified', function(e) {
      if ( e.attrName != 'value' ) {
        return ;
      }
      observerCallback( [e] );
    }, false);
  }
};

SubscriptionWidget.prototype.fetchNextItems = function(){
  this.domElements.loading.classList.add('active');
  
  (function(self, searchPattern) {
  
    var onRestResponse = function(error, calendars){
      
      if ( searchPattern !== self.getSearchPattern() ) {
        UO_logger.debug("User changed search pattern, ignoring REST response");
        return;
      }
      
      if (error) {
        UO_logger.error(error);
        self.domElements.error.removeAttribute('hidden');
        return;
      }
  
      if(self.paginer.endOfResult){
        UO_logger.debug("paginer.endOfResult : " + self.paginer.endOfResult, false, false);
        self.treeWidget.unbind();
      }
  
      var userCalendars = self.treeRowBuilder(calendars);
      self.treeWidget.addChildren(userCalendars);
      self.domElements.loading.classList.remove('active');
  
    };
    
    self.paginer.fetchNextItems(onRestResponse);
    
  })(this, this.getSearchPattern());
};

SubscriptionWidget.prototype.initSearch = function(pattern) {
  this.paginer.pattern = pattern;
  this.fetchNextItems();
  this.treeWidget.cleanTree();
  this.treeWidget.bind();
};

SubscriptionWidget.prototype.getSearchPattern = function() {
  return this.domElements.searchField.value;
};

SubscriptionWidget.prototype.search = function() {
  var pattern = this.getSearchPattern();
  if ( pattern === this.paginer.pattern ) {
    return false;
  }
  this.initSearch(pattern);
};